# -*- coding: utf-8 -*-
from django.contrib import admin

# Register your models here.
from .models import *

admin.site.register((Messaging, FBUserProfile, UserLocation, Greetings, GetStartedButton, PersistentMenu))



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from django.utils.translation import ugettext_lazy as _
import requests
from djmessenger.settings import DJM_THREAD_SETTINGS_URL
from djmessenger.models import PersistentMenu


class Command(BaseCommand):
    def handle(self, *args, **options):
        if PersistentMenu.objects.count() == 0:
            print(_('You did not add any persistent menu item into database'))
            return
        menus = PersistentMenu.objects.all()

        actions = []
        for menu in menus:
            actions.append(menu.json())
        data = {
            "setting_type": "call_to_actions",
            "thread_state": "existing_thread",
            "call_to_actions": actions
        }
        status = requests.post(DJM_THREAD_SETTINGS_URL,
                               headers={"Content-Type": "application/json"},
                               json=data)
        print(status.content)




# -*- coding: utf-8 -*-
"""
Similarly, this sending module is to represent the json templates that should
be used when sending something back to the user based on Facebook reference:

https://developers.facebook.com/docs/messenger-platform/send-api-reference

## CommonSender

`CommonSender` defines 4 attributes: `recipient`, `sender_action`,
`notification_type`, `message` because that's what [Facebook Send API](https://developers.facebook.com/docs/messenger-platform/send-api-reference) requires

And `send()` method actually sends the json string (serialized from the above
4 attributes) to Facebook endpoint using `requests` module

Each sender will definitely get psid as its first constructor argument. You can
define your own sender to provide customized behavior and then use it in the
routing policy

## Provided Senders

### ButtonSender (TBD)

### DefaultSender

Simple sends back the text defined in `DJM_DEFAULT_SENDER_TEXT`

- args: takes no argument

### MultimediaSender

Sends back a `SendingType` with its url

    ```
    image
    audio
    video
    file
    ```

- args:
    - sending_type: image, audio, video, file
    - url: url to the resource

### QuickReplySender (TBD)

### SenderActionSender

Sends back a sender action, check [Facebook Reference](https://developers.facebook.com/docs/messenger-platform/send-api-reference/sender-actions)

- args:
    - action: mark_seen, typing_on, typing_off

### SimpleMessageSender

Sends back a simple text message, could be i18n'ed

- args:
    - text: message to send back to user

"""
import requests
from abc import abstractmethod
import logging
import gettext

from djmessenger.utils.serializable import SerializableEnum, Serializable
from djmessenger.receiving import Recipient
from djmessenger.settings import DJM_POST_MESSAGE_URL, DJM_DEFAULT_SENDER_TEXT, DJM_BOT_PREFIX
from djmessenger.models import Sending, FBUserProfile
# from django.utils.translation import ugettext_lazy as _
from djmessenger.utils.i18n import *


logger = logging.getLogger(__name__)


class SendingType(SerializableEnum):
    pass


SendingType.SIMPLE_TEXT = SendingType('simple_text')
SendingType.QUICK_REPLY = SendingType('quick_reply')
SendingType.BUTTON = SendingType('button')
SendingType.IMAGE = SendingType('image')
SendingType.AUDIO = SendingType('audio')
SendingType.VIDEO = SendingType('video')
SendingType.FILE = SendingType('file')
SendingType.SENDER_ACTION = SendingType('sender_action')


class NotificationType(SerializableEnum):
    pass


NotificationType.REGULAR = NotificationType('REGULAR')
NotificationType.SILEN_PUSH = NotificationType('SILEN_PUSH')
NotificationType.NO_PUSH = NotificationType('NO_PUSH')


class SenderAction(SerializableEnum):
    pass


SenderAction.MARK_SEEN = SenderAction('mark_seen')
SenderAction.TYPING_ON = SenderAction('typing_on')
SenderAction.TYPING_OFF = SenderAction('typing_off')


class CommonSender(Serializable):
    includes = ('recipient', 'sender_action', 'notification_type', 'message')

    def __init__(self, psid, sender_action=None,
                 notification_type=NotificationType.REGULAR.name):
        """
        Define highest level of sending template

        @param psid: the user you'd like to send this to, this psid must be
                     obtained from receiving message
        @type psid: str

        @param sender_action: sender action
        @type sender_action: SenderAction

        @param notification_type: mostly no need to change this, default is
                                  regular
        @type notification_type: NotificationType
        """
        self.recipient = Recipient(psid)
        self.sender_action = sender_action
        self.notification_type = notification_type
        self.message = None

    @abstractmethod
    def get_message(self):
        """
        Based on the attributes the class constructor constructs, this method
        returns the message body, the returned must be a dict and valid json
        object

        @return:
        @rtype: dict
        """
        pass

    def send(self):
        self.message = self.get_message()
        data = self.json()
        logger.debug('Sending %s message to user %s: %s' %
                     (self.get_sending_type(), self.recipient.id, data))
        try:
            recipient = FBUserProfile.objects.get(pk=self.recipient.id)
        except FBUserProfile.DoesNotExist:
            pass
        try:
            sending = Sending.objects.create(recipient=recipient, data=data,
                                             type=self.get_sending_type)
        except Exception as e:
            logger.debug('Failed to create Sending object because %s' % e)
        status = requests.post(DJM_POST_MESSAGE_URL,
                               headers={"Content-Type": "application/json"},
                               json=data)
        sending.response = status.content
        sending.status_code = status.status_code
        sending.save()

    @abstractmethod
    def get_sending_type(self):
        """
        Return the SendingType that this Sender sends

        @return:
        @rtype: SendingType
        """
        pass

    @classmethod
    def preprocess_outgoing_string(cls, psid, text):
        """This method take cares of any preprocessing of the outgoing text

        @param psid:
        @param text:
        @return:
        """
        install_user_locale(psid)
        try:
            ret = _(DJM_BOT_PREFIX).encode('utf-8') + _(text).encode('utf-8')
        except:
            # if the locale is not supported, _() will fail, so catch it and
            # fallback to English
            ret = DJM_BOT_PREFIX.encode('utf-8') + text.encode('utf-8')
        reset_locale()
        return ret

    def get_psid(self):
        return self.recipient.id


class SenderActionSender(CommonSender):
    """
    Send a Sender Action
    """
    def __init__(self, psid, action):
        super().__init__(psid, action)

    def get_sending_type(self):
        return SendingType.SENDER_ACTION

    def get_message(self):
        # no message
        return ""


class SimpleMessageSender(CommonSender):
    """
    Send a simple text message
    """
    def __init__(self, psid, text):
        super().__init__(psid)
        self.text = text

    def get_message(self):
        # FUTURE: we can probably move this part to CommonSender so that
        #         subclasses don't need to worry about it
        text = CommonSender.preprocess_outgoing_string(
            self.get_psid(), self.text)
        if isinstance(text, bytes):
            text = text.decode('utf-8')
        return {"text": text}

    def get_sending_type(self):
        return SendingType.SIMPLE_TEXT


class DefaultSender(SimpleMessageSender):
    """
    Sends back simple text DJM_DEFAULT_SENDER_TEXT
    """
    def __init__(self, psid):
        super().__init__(psid, DJM_DEFAULT_SENDER_TEXT)


class MultimediaSender(CommonSender):
    # TODO: should work with django static???
    """
    Send back multimedia from a url

    SendingType.IMAGE = SendingType('IMAGE')
    SendingType.AUDIO = SendingType('AUDIO')
    SendingType.VIDEO = SendingType('VIDEO')
    SendingType.FILE = SendingType('FILE')
    """

    def __init__(self, psid, sending_type, url):
        super().__init__(psid)
        self.sending_type = sending_type
        self.url = url

    def get_sending_type(self):
        return self.sending_type

    def get_message(self):
        return {"attachment": {
            "type": self.get_sending_type().name.lower(),
            "payload": {
                "url": self.url
            }
        }}

class ButtonSender(CommonSender):
    # TODO:
    pass


class QuickReplySender(CommonSender):
    # TODO:
    pass



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from djmessenger.models import GetStartedButton
import requests
from djmessenger.settings import DJM_THREAD_SETTINGS_URL
from django.utils.translation import ugettext_lazy as _


class Command(BaseCommand):
    def handle(self, *args, **options):
        if GetStartedButton.objects.count() == 0:
            print(_('You did not add any get started button payload into database'))
            return
        payload = GetStartedButton.objects.first().payload
        data = {
            "setting_type": "call_to_actions",
            "thread_state": "new_thread",
            "call_to_actions": [
                {
                    "payload": payload
                }
            ]
        }
        status = requests.post(DJM_THREAD_SETTINGS_URL,
                               headers={"Content-Type": "application/json"},
                               json=data)
        print(status.content)



# -*- coding: utf-8 -*-
import requests
import logging

from django.db import models
from django.utils.translation import ugettext_lazy as _

from djmessenger.utils.serializable import Serializable
from djmessenger.exceptions import DJMInvalidConfigException


logger = logging.getLogger(__name__)


class Messaging(models.Model):
    body = models.TextField()
    type = models.CharField(max_length=128, null=True, blank=True)

    class Meta:
        db_table = 'djm_messaging'


class Sending(models.Model):
    recipient = models.ForeignKey('FBUserProfile')
    date_sent = models.DateTimeField(auto_now_add=True)
    data = models.TextField()
    type = models.CharField(max_length=128, null=True, blank=True)
    response = models.TextField(null=True, blank=True)
    status_code = models.IntegerField(null=True, blank=True)

    class Meta:
        db_table = 'djm_sending'


class FBUserProfile(models.Model):
    """
    This FacebookUser class is to represent a user that sent a message to us
    via Facebook Messenger, from the message relayed by Facebook, we can look
    up user details which will be stored here
    """
    psid = models.CharField(max_length=512, primary_key=True)
    first_name = models.CharField('first name', max_length=512, null=True,
                                  blank=True)
    last_name = models.CharField('last name', max_length=512, null=True,
                                 blank=True)
    profile_pic = models.TextField(null=True, blank=True)
    locale = models.CharField(max_length=128, null=True, blank=True)
    timezone = models.SmallIntegerField()
    gender = models.CharField(max_length=20, null=True, blank=True)
    is_active = models.BooleanField(default=True)
    date_joined = models.DateTimeField(auto_now_add=True)
    thumbups = models.BigIntegerField(default=0)

    class Meta:
        db_table = 'djm_user'


class UserLocation(models.Model):
    """
    From the message user sent via Facebook to us, addition to saving the raw
    request body into RawBody, we try to see if the sent message is actually a
    location that contains lat and long, if yes, then we save it here
    """
    user = models.ForeignKey(FBUserProfile, null=False, blank=False)
    latitude = models.DecimalField(max_digits=10,
                                   decimal_places=8,
                                   null=True, blank=True)
    longitude = models.DecimalField(max_digits=11,
                                    decimal_places=8,
                                    null=True, blank=True)
    timestamp = models.BigIntegerField(
        help_text='Facebook returns timestamp as EPOCH')
    url = models.URLField(max_length=1000, null=True, blank=True)
    mid = models.CharField(max_length=512, null=True, blank=True)
    seq = models.IntegerField(blank=True, null=True)
    date_created = models.DateTimeField(auto_now_add=True)

    # def save(self, *args, **kwargs):
    #     self.date_created = datetime.utcnow().replace(tzinfo=utc)
    #     super().save(*args, **kwargs)

    class Meta:
        get_latest_by = 'date_created'
        db_table = 'djm_userlocation'

    def __str__(self):
        return repr(self)

    def __repr__(self):
        return '%s(%s, %s)' % (self.user, self.latitude, self.longitude)


class Greetings(models.Model):
    text = models.CharField(
        max_length=160,
        unique=True,
        null=False,
        blank=False,
        primary_key=True,
        help_text=_('This is the greeting text your user will see when the user'
                    'interacts with your page in the first time on Messenger'
                    '. You can only configure 1 of this, text length limit is'
                    '160 chars.'))

    def save(self, *args, **kwargs):
        if Greetings.objects.count() > 0:
            raise DJMInvalidConfigException(
                _('Facebook regulated that greeting text can have only 1, it'
                  'seems you are trying to add more than 1'))
        super().save(*args, **kwargs)

    class Meta:
        db_table = 'djm_greetings'


class GetStartedButton(models.Model):
    """
    The Get Started button is only rendered the first time the user interacts
    with a the Page on Messenger.
    """
    payload = models.CharField(
        max_length=160,
        unique=True,
        null=False,
        blank=False,
        primary_key=True,
        help_text=_('The payload string will be sent back to you via Postback,'
                    'so once you added a Get Started Button here, you also '
                    'need to define how to handle the payload'))

    def save(self, *args, **kwargs):
        if GetStartedButton.objects.count() > 0:
            raise DJMInvalidConfigException(
                _('Facebook regulated that get started button can have only 1, '
                  'it seems you are trying to add more than 1'))
        super().save(*args, **kwargs)

    class Meta:
        db_table = 'djm_getstarted'


class PersistentMenu(models.Model, Serializable):
    """
    The Persistent Menu is a menu that is always available to the user.
    This menu should contain top-level actions that users can enact at any
    point. Having a persistent menu easily communicates the basic capabilities
    of your bot for first-time and returning users.
    The menu can be invoked by a user, by tapping on the 3-caret icon on the
    left of the composer.
    """
    MENU_TYPE_CHOICES = (
        ('postback', _('postback')),
        ('web_url', _('web_url'))
    )
    type = models.CharField(
        max_length=10, choices=MENU_TYPE_CHOICES, default='web_url',
        help_text=_('postback means when clicking on the menu, it will send '
                    'the payload back to the server; while web_url simply opens'
                    ' the url'))
    title = models.CharField(max_length=30, null=False, blank=False, unique=True
                             , help_text=_('Button title, case sensitive, '
                                           'unique'))
    url = models.URLField(
        help_text=_('For web_url buttons, this URL is opened in a mobile '
                    'browser when the button is tapped')
    )
    payload = models.CharField(
        max_length=1000,
        help_text=_('For postback buttons, this data will be sent back to you '
                    'via webhook')
    )

    def save(self, *args, **kwargs):
        if PersistentMenu.objects.count() == 5:
            raise DJMInvalidConfigException(
                _('Facebook regulated that get started button can have only 5, '
                  'it seems you are trying to add more than 1'))
        if self.type == 'postback' and not self.payload:
            raise ValueError(_('If persistent menu type is postback, you need'
                               ' to configure payload as well'))
        if self.type == 'web_url' and not self.url:
            raise ValueError(_('If persisent menu type is web_url, you need to '
                               'configure url as well'))
        super().save(*args, **kwargs)

    class Meta:
        db_table = 'djm_persistentmenu'

    def json(self):
        if self.type == 'postback':
            return {
                'type': self.type,
                'title': self.title,
                'payload': self.payload
            }
        elif self.type == 'web_url':
            return {
                'type': self.type,
                'title': self.title,
                'url': self.url
            }



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
import requests
from djmessenger.settings import DJM_THREAD_SETTINGS_URL
from djmessenger.models import Greetings
from django.utils.translation import ugettext_lazy as _


class Command(BaseCommand):
    def handle(self, *args, **options):
        if Greetings.objects.count() == 0:
            print(_('You did not add any greetings text into database'))
            return
        text = Greetings.objects.first().text
        data = {
            "setting_type": "greeting",
            "greeting": {
                "text": text
            }
        }
        status = requests.post(DJM_THREAD_SETTINGS_URL,
                               headers={"Content-Type": "application/json"},
                               json=data)
        print(status.content)

